@@ -78,7 +78,9 @@ def get_longest_path(data,seed_edge):
         keep_digging = []
         for edge in temp_edges:
             if data[edge,15] > -1:
-                temp_edges.extend([int(data[edge,15]), int(data[edge,16])])
+                temp_edges.extend([int(data[edge,15])])
+                if data[edge,16] > -1:
+                    temp_edges.extend([int(data[edge,16])])
                 temp_edges.remove(edge)
                 keep_digging.append(True)
             else:
@@ -93,16 +95,18 @@ def get_longest_path(data,seed_edge):
     max_edge_depths = [i for i,j in enumerate(edge_depths) if j == max_depth]
     paths = [[temp_edges[i]] for i in max_edge_depths]
     path_lengths = [data[i[0],20] for i in paths]
-    retrace = True
-    while retrace:
+    retrace = [True]*len(paths)
+    while np.any(retrace):
         for i, path in enumerate(paths):
-            path.insert(0,int(data[path[0],17]))
-            path_lengths[i] += data[path[0],20]
-        if paths[0][0] == seed_edge:
-            retrace = False
+            if retrace[i]:
+                path.insert(0,int(data[path[0],17]))
+                path_lengths[i] += data[path[0],20]
+                if paths[i][0] == seed_edge:
+                    retrace[i] = False
     return paths[path_lengths.index(max(path_lengths))]
 
 def get_alternate_path(data,seed_edge,reference=None):
+    print('reference: {}'.format(reference))
     if reference is None:
         reference = get_longest_path(data,seed_edge)
     else:
@@ -131,6 +135,19 @@ def get_branches(data):
     counter = [len(path[:-1])]
     idx = 0
     while len(upcoming_evaluations) > 0:
+        print(upcoming_evaluations)
+        if data[upcoming_evaluations[-1],15] > -1 and data[upcoming_evaluations[-1],16] > -1:
+            pass
+        else:
+            upcoming_evaluations.pop(-1)
+            counter[idx] -= 1
+            if counter[idx] == 0:
+                counter[idx] = None
+                for i in reversed(range(len(counter))):
+                    if counter[i] is not None:
+                        idx = i
+                        break
+            continue
         path = get_alternate_path(data,upcoming_evaluations.pop(-1),reference=branches[idx])
         counter[idx] -= 1
         if counter[idx] == 0:
